@@ -1,18 +1,5 @@
-// ======================================================================== //
-// Copyright 2009-2018 Intel Corporation                                    //
-//                                                                          //
-// Licensed under the Apache License, Version 2.0 (the "License");          //
-// you may not use this file except in compliance with the License.         //
-// You may obtain a copy of the License at                                  //
-//                                                                          //
-//     http://www.apache.org/licenses/LICENSE-2.0                           //
-//                                                                          //
-// Unless required by applicable law or agreed to in writing, software      //
-// distributed under the License is distributed on an "AS IS" BASIS,        //
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. //
-// See the License for the specific language governing permissions and      //
-// limitations under the License.                                           //
-// ======================================================================== //
+// Copyright 2009-2021 Intel Corporation
+// SPDX-License-Identifier: Apache-2.0
 
 #include "bvh_intersector_hybrid.h"
 #include "bvh_traverser1.h"
@@ -64,7 +51,8 @@ namespace embree
       stack[0].dist = neg_inf;
 
       /* load the ray into SIMD registers */
-      TravRay<N,Nx,robust> tray1(k, tray.org, tray.dir, tray.rdir, tray.nearXYZ, tray.tnear[k], tray.tfar[k]);
+      TravRay<N,robust> tray1;
+      tray1.template init<K>(k, tray.org, tray.dir, tray.rdir, tray.nearXYZ, tray.tnear[k], tray.tfar[k]);
 
       /* pop loop */
       while (true) pop:
@@ -75,33 +63,24 @@ namespace embree
         NodeRef cur = NodeRef(stackPtr->ptr);
 
         /* if popped node is too far, pop next one */
-#if defined(__AVX512ER__)
-        /* much faster on KNL */
-        if (unlikely(any(vfloat<Nx>(*(float*)&stackPtr->dist) > tray1.tfar)))
-          continue;
-#else
         if (unlikely(*(float*)&stackPtr->dist > ray.tfar[k]))
           continue;
-#endif
 
         /* downtraversal loop */
         while (true)
         {
-          /* stop if we found a leaf node */
-          if (unlikely(cur.isLeaf())) break;
-          STAT3(normal.trav_nodes, 1, 1, 1);
-
           /* intersect node */
-          size_t mask = 0;
-          vfloat<Nx> tNear;
-          BVHNNodeIntersector1<N, Nx, types, robust>::intersect(cur, tray1, ray.time()[k], tNear, mask);
+          size_t mask; vfloat<N> tNear;
+          STAT3(normal.trav_nodes, 1, 1, 1);
+          bool nodeIntersected = BVHNNodeIntersector1<N, types, robust>::intersect(cur, tray1, ray.time()[k], tNear, mask);
+          if (unlikely(!nodeIntersected)) { STAT3(normal.trav_nodes,-1,-1,-1); break; }
 
           /* if no child is hit, pop next node */
           if (unlikely(mask == 0))
             goto pop;
 
           /* select next child and push other children */
-          BVHNNodeTraverser1Hit<N, Nx, types>::traverseClosestHit(cur, mask, tNear, stackPtr, stackEnd);
+          BVHNNodeTraverser1Hit<N, types>::traverseClosestHit(cur, mask, tNear, stackPtr, stackEnd);
         }
 
         /* this is a leaf node */
@@ -251,7 +230,7 @@ namespace embree
             continue;
 
           /* switch to single ray traversal */
-#if (!defined(__WIN32__) || defined(__X86_64__)) && defined(__SSE4_2__)
+#if (!defined(__WIN32__) || defined(__X86_64__)) && ((defined(__aarch64__)) || defined(__SSE4_2__))
 #if FORCE_SINGLE_MODE == 0
           if (single)
 #endif
@@ -276,7 +255,7 @@ namespace embree
             const vbool<K> valid_node = tray.tfar > curDist;
             STAT3(normal.trav_nodes, 1, popcnt(valid_node), K);
             const NodeRef nodeRef = cur;
-            const BaseNode* __restrict__ const node = nodeRef.baseNode(types);
+            const BaseNode* __restrict__ const node = nodeRef.baseNode();
 
             /* set cur to invalid */
             cur = BVH::emptyNode;
@@ -431,7 +410,8 @@ namespace embree
         tray.tnear = select(octant_valid, org_ray_tnear, vfloat<K>(pos_inf));
         tray.tfar  = select(octant_valid, org_ray_tfar , vfloat<K>(neg_inf));
 
-        Frustum<robust> frustum(octant_valid, tray.org, tray.rdir, tray.tnear, tray.tfar, N);
+        Frustum<robust> frustum;
+        frustum.template init<K>(octant_valid, tray.org, tray.rdir, tray.tnear, tray.tfar, N);
 
         StackItemT<NodeRef> stack[stackSizeSingle];  // stack of nodes
         StackItemT<NodeRef>* stackPtr = stack + 1;   // current stack pointer
@@ -456,10 +436,10 @@ namespace embree
             /* process nodes */
             //STAT3(normal.trav_nodes, 1, popcnt(valid_node), K);
             const NodeRef nodeRef = cur;
-            const AlignedNode* __restrict__ const node = nodeRef.alignedNode();
+            const AABBNode* __restrict__ const node = nodeRef.getAABBNode();
 
-            vfloat<Nx> fmin;
-            size_t m_frustum_node = intersectNodeFrustum<N,Nx>(node, frustum, fmin);
+            vfloat<N> fmin;
+            size_t m_frustum_node = intersectNodeFrustum<N>(node, frustum, fmin);
 
             if (unlikely(!m_frustum_node)) goto pop;
             cur = BVH::emptyNode;
@@ -480,7 +460,7 @@ namespace embree
               {                                
                 const vfloat<K> childDist = fmin[i];
                 const NodeRef child = node->child(i);
-                child.prefetch();
+                BVHN<N>::prefetch(child);
                 if (any(childDist < curDist))
                 {
                   if (likely(cur != BVH::emptyNode)) {
@@ -534,7 +514,7 @@ namespace embree
           if (likely(any((ray.tfar < tray.tfar) & valid_leaf)))
           {
             tray.tfar = select(valid_leaf, ray.tfar, tray.tfar);
-            frustum.updateMaxDist(tray.tfar);
+            frustum.template updateMaxDist<K>(tray.tfar);
           }
 
           if (unlikely(lazy_node)) {
@@ -568,7 +548,8 @@ namespace embree
         stack[0] = root;
 
         /* load the ray into SIMD registers */
-        TravRay<N,Nx,robust> tray1(k, tray.org, tray.dir, tray.rdir, tray.nearXYZ, tray.tnear[k], tray.tfar[k]);
+        TravRay<N,robust> tray1;
+        tray1.template init<K>(k, tray.org, tray.dir, tray.rdir, tray.nearXYZ, tray.tnear[k], tray.tfar[k]);
 
 	/* pop loop */
 	while (true) pop:
@@ -581,21 +562,18 @@ namespace embree
           /* downtraversal loop */
           while (true)
           {
-            /* stop if we found a leaf node */
-            if (unlikely(cur.isLeaf())) break;
-            STAT3(shadow.trav_nodes, 1, 1, 1);
-
             /* intersect node */
-            size_t mask = 0;
-            vfloat<Nx> tNear;
-            BVHNNodeIntersector1<N, Nx, types, robust>::intersect(cur, tray1, ray.time()[k], tNear, mask);
+            size_t mask; vfloat<N> tNear;
+            STAT3(shadow.trav_nodes, 1, 1, 1);
+            bool nodeIntersected = BVHNNodeIntersector1<N, types, robust>::intersect(cur, tray1, ray.time()[k], tNear, mask);
+            if (unlikely(!nodeIntersected)) { STAT3(shadow.trav_nodes,-1,-1,-1); break; }
 
             /* if no child is hit, pop next node */
             if (unlikely(mask == 0))
               goto pop;
 
             /* select next child and push other children */
-            BVHNNodeTraverser1Hit<N, Nx, types>::traverseAnyHit(cur, mask, tNear, stackPtr, stackEnd);
+            BVHNNodeTraverser1Hit<N, types>::traverseAnyHit(cur, mask, tNear, stackPtr, stackEnd);
           }
 
           /* this is a leaf node */
@@ -698,7 +676,7 @@ namespace embree
           continue;
 
         /* switch to single ray traversal */
-#if (!defined(__WIN32__) || defined(__X86_64__)) && defined(__SSE4_2__)
+#if (!defined(__WIN32__) || defined(__X86_64__)) && ((defined(__aarch64__)) || defined(__SSE4_2__))
 #if FORCE_SINGLE_MODE == 0
         if (single)
 #endif
@@ -726,7 +704,7 @@ namespace embree
           const vbool<K> valid_node = tray.tfar > curDist;
           STAT3(shadow.trav_nodes, 1, popcnt(valid_node), K);
           const NodeRef nodeRef = cur;
-          const BaseNode* __restrict__ const node = nodeRef.baseNode(types);
+          const BaseNode* __restrict__ const node = nodeRef.baseNode();
 
           /* set cur to invalid */
           cur = BVH::emptyNode;
@@ -845,7 +823,8 @@ namespace embree
         tray.tnear = select(octant_valid, org_ray_tnear, vfloat<K>(pos_inf));
         tray.tfar  = select(octant_valid, org_ray_tfar,  vfloat<K>(neg_inf));
 
-        const Frustum<robust> frustum(octant_valid, tray.org, tray.rdir, tray.tnear, tray.tfar, N);
+        Frustum<robust> frustum;
+        frustum.template init<K>(octant_valid, tray.org, tray.rdir, tray.tnear, tray.tfar, N);
 
         StackItemMaskT<NodeRef> stack[stackSizeSingle];  // stack of nodes
         StackItemMaskT<NodeRef>* stackPtr = stack + 1;   // current stack pointer
@@ -870,10 +849,10 @@ namespace embree
             /* process nodes */
             //STAT3(normal.trav_nodes, 1, popcnt(valid_node), K);
             const NodeRef nodeRef = cur;
-            const AlignedNode* __restrict__ const node = nodeRef.alignedNode();
+            const AABBNode* __restrict__ const node = nodeRef.getAABBNode();
 
-            vfloat<Nx> fmin;
-            size_t m_frustum_node = intersectNodeFrustum<N,Nx>(node, frustum, fmin);
+            vfloat<N> fmin;
+            size_t m_frustum_node = intersectNodeFrustum<N>(node, frustum, fmin);
 
             if (unlikely(!m_frustum_node)) goto pop;
             cur = BVH::emptyNode;
@@ -894,7 +873,7 @@ namespace embree
               {                                
                 const NodeRef child = node->child(i);
                 assert(child != BVH::emptyNode);
-                child.prefetch();
+                BVHN<N>::prefetch(child);
                 if (likely(cur != BVH::emptyNode)) {
                   num_child_hits++;
                   stackPtr->ptr  = cur;
