@@ -3,13 +3,14 @@
 
 #pragma once
 
+#include "../../../include/embree3/rtcore.h"
 #include "../math/vec.h"
 
 namespace embree {
 
 struct DifferentialGeometry
 {
-  unsigned int instID;
+  unsigned int instIDs[RTC_MAX_INSTANCE_LEVEL_COUNT];
   unsigned int geomID;
   unsigned int primID;
   float u,v;
