@@ -229,6 +229,8 @@ namespace embree
     Ref<SceneGraph::Node> loadTriangleLight(const Ref<XML>& xml);
     Ref<SceneGraph::Node> loadQuadLight(const Ref<XML>& xml);
     Ref<SceneGraph::Node> loadHDRILight(const Ref<XML>& xml);
+    Ref<SceneGraph::Node> loadLight(const Ref<XML>& xml);
+    Ref<SceneGraph::Node> loadAnimatedLight(const Ref<XML>& xml);
     
     Ref<SceneGraph::Node> loadTriangleMesh(const Ref<XML>& xml);
     Ref<SceneGraph::Node> loadQuadMesh(const Ref<XML>& xml);
@@ -240,6 +242,7 @@ namespace embree
  
   private:
     Ref<SceneGraph::Node> loadPerspectiveCamera(const Ref<XML>& xml);
+    Ref<SceneGraph::Node> loadAnimatedPerspectiveCamera(const Ref<XML>& xml);
     Ref<SceneGraph::MaterialNode> loadMaterial(const Ref<XML>& xml);
     Ref<SceneGraph::Node> loadTransformNode(const Ref<XML>& xml);
     Ref<SceneGraph::Node> loadMultiTransformNode(const Ref<XML>& xml);
@@ -662,8 +665,8 @@ namespace embree
     const AffineSpace3fa space = load<AffineSpace3fa>(xml->child("AffineSpace"));
     const Vec3fa I = load<Vec3f>(xml->child("I"));
     const Vec3fa P = Vec3fa(zero);
-    const Ref<SceneGraph::Light> light = new SceneGraph::PointLight(P,I);
-    return new SceneGraph::LightNode(light->transform(space));
+    const SceneGraph::PointLight light = SceneGraph::PointLight(P,I);
+    return new SceneGraph::LightNodeImpl<SceneGraph::PointLight>(light.transform(space));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadSpotLight(const Ref<XML>& xml) 
@@ -674,8 +677,8 @@ namespace embree
     const Vec3fa D = Vec3fa(0,0,1);
     const float angleMin = load<float>(xml->child("angleMin"));
     const float angleMax = load<float>(xml->child("angleMax"));
-    const Ref<SceneGraph::Light> light = new SceneGraph::SpotLight(P,D,I,angleMin,angleMax);
-    return new SceneGraph::LightNode(light->transform(space));
+    const SceneGraph::SpotLight light = SceneGraph::SpotLight(P,D,I,angleMin,angleMax);
+    return new SceneGraph::LightNodeImpl<SceneGraph::SpotLight>(light.transform(space));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadDirectionalLight(const Ref<XML>& xml) 
@@ -683,8 +686,8 @@ namespace embree
     const AffineSpace3fa space = load<AffineSpace3fa>(xml->child("AffineSpace"));
     const Vec3fa E = load<Vec3fa>(xml->child("E"));
     const Vec3fa D = Vec3fa(0,0,1);
-    const Ref<SceneGraph::Light> light = new SceneGraph::DirectionalLight(D,E);
-    return new SceneGraph::LightNode(light->transform(space));
+    const SceneGraph::DirectionalLight light = SceneGraph::DirectionalLight(D,E);
+    return new SceneGraph::LightNodeImpl<SceneGraph::DirectionalLight>(light.transform(space));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadDistantLight(const Ref<XML>& xml) 
@@ -693,14 +696,14 @@ namespace embree
     const Vec3fa L = load<Vec3fa>(xml->child("L"));
     const Vec3fa D = Vec3fa(0,0,1);
     const float halfAngle = load<float>(xml->child("halfAngle"));
-    const Ref<SceneGraph::Light> light = new SceneGraph::DistantLight(D,L,halfAngle);
-    return new SceneGraph::LightNode(light->transform(space));
+    const SceneGraph::DistantLight light = SceneGraph::DistantLight(D,L,halfAngle);
+    return new SceneGraph::LightNodeImpl<SceneGraph::DistantLight>(light.transform(space));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadAmbientLight(const Ref<XML>& xml) 
   {
     const Vec3fa L = load<Vec3fa>(xml->child("L"));
-    return new SceneGraph::LightNode(new SceneGraph::AmbientLight(L));
+    return new SceneGraph::LightNodeImpl<SceneGraph::AmbientLight>(SceneGraph::AmbientLight(L));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadTriangleLight(const Ref<XML>& xml) 
@@ -710,7 +713,7 @@ namespace embree
     const Vec3fa v0 = xfmPoint(space, Vec3fa(1, 0, 0));
     const Vec3fa v1 = xfmPoint(space, Vec3fa(0, 1, 0));
     const Vec3fa v2 = xfmPoint(space, Vec3fa(0, 0, 0));
-    return new SceneGraph::LightNode(new SceneGraph::TriangleLight(v0,v1,v2,L));
+    return new SceneGraph::LightNodeImpl<SceneGraph::TriangleLight>(SceneGraph::TriangleLight(v0,v1,v2,L));
   }
 
   Ref<SceneGraph::Node> XMLLoader::loadQuadLight(const Ref<XML>& xml) 
@@ -721,7 +724,43 @@ namespace embree
     const Vec3fa v1 = xfmPoint(space, Vec3fa(0, 1, 0));
     const Vec3fa v2 = xfmPoint(space, Vec3fa(1, 1, 0));
     const Vec3fa v3 = xfmPoint(space, Vec3fa(1, 0, 0));
-    return new SceneGraph::LightNode(new SceneGraph::QuadLight(v0,v1,v2,v3,L));
+    return new SceneGraph::LightNodeImpl<SceneGraph::QuadLight>(SceneGraph::QuadLight(v0,v1,v2,v3,L));
+  }
+
+   Ref<SceneGraph::Node> XMLLoader::loadLight(const Ref<XML>& xml) 
+   {
+     const std::string id = xml->parm("id");
+     if      (xml->name == "PointLight"      ) return state.sceneMap[id] = loadPointLight      (xml);
+     else if (xml->name == "SpotLight"       ) return state.sceneMap[id] = loadSpotLight       (xml);
+     else if (xml->name == "DirectionalLight") return state.sceneMap[id] = loadDirectionalLight(xml);
+     else if (xml->name == "DistantLight"    ) return state.sceneMap[id] = loadDistantLight    (xml);
+     else if (xml->name == "AmbientLight"    ) return state.sceneMap[id] = loadAmbientLight    (xml);
+     else if (xml->name == "TriangleLight"   ) return state.sceneMap[id] = loadTriangleLight   (xml);
+     else if (xml->name == "AnimatedLight"   ) return state.sceneMap[id] = loadAnimatedLight   (xml);
+     else if (xml->name == "QuadLight"       ) return state.sceneMap[id] = loadQuadLight       (xml);
+     else THROW_RUNTIME_ERROR(xml->loc.str()+": invalid light node: "+xml->name);
+   }
+  
+  Ref<SceneGraph::Node> XMLLoader::loadAnimatedLight(const Ref<XML>& xml) 
+  {
+    size_t numLights = xml->size();
+    if (numLights == 0)
+      return nullptr;
+
+    /* load list of lights */
+    std::vector<Ref<SceneGraph::LightNode>> lights(numLights);
+    for (size_t i=0; i<numLights; i++) 
+      lights[i] = loadLight(xml->child(i)).dynamicCast<SceneGraph::LightNode>();
+
+    /* check that all lights are of same type */
+    auto light_type = lights[0]->getType();
+    for (size_t i=1; i<numLights; i++) {
+      if (light_type != lights[i]->getType())
+        THROW_RUNTIME_ERROR(xml->loc.str()+": light types do not match");
+    }
+
+    const Vec2f time_range = xml->parm_Vec2f("time_range");
+    return new SceneGraph::AnimatedLightNode(std::move(lights),BBox1f(time_range.x,time_range.y));
   }
 
   std::shared_ptr<Texture> XMLLoader::loadTextureParm(const Ref<XML>& xml)
@@ -765,6 +804,21 @@ namespace embree
     return new SceneGraph::PerspectiveCameraNode(from,to,up,fov);
   }
 
+  Ref<SceneGraph::Node> XMLLoader::loadAnimatedPerspectiveCamera(const Ref<XML>& xml) 
+  {
+    size_t numCameras = xml->size();
+    if (numCameras == 0)
+      return nullptr;
+    
+    std::vector<Ref<SceneGraph::PerspectiveCameraNode>> cameras(numCameras);
+    
+    for (size_t i=0; i<numCameras; i++) 
+      cameras[i] = loadPerspectiveCamera(xml->child(i)).dynamicCast<SceneGraph::PerspectiveCameraNode>();
+
+    const Vec2f time_range = xml->parm_Vec2f("time_range");
+    return new SceneGraph::AnimatedPerspectiveCameraNode(std::move(cameras),BBox1f(time_range.x,time_range.y));
+  }
+
   Parms XMLLoader::loadMaterialParms(const Ref<XML>& parms)
   {
     Parms material;
@@ -1378,6 +1432,7 @@ namespace embree
       else if (xml->name == "DistantLight"    ) node = state.sceneMap[id] = loadDistantLight    (xml);
       else if (xml->name == "AmbientLight"    ) node = state.sceneMap[id] = loadAmbientLight    (xml);
       else if (xml->name == "TriangleLight"   ) node = state.sceneMap[id] = loadTriangleLight   (xml);
+      else if (xml->name == "AnimatedLight"   ) node = state.sceneMap[id] = loadAnimatedLight   (xml);
       else if (xml->name == "QuadLight"       ) node = state.sceneMap[id] = loadQuadLight       (xml);
       else if (xml->name == "TriangleMesh"    ) node = state.sceneMap[id] = loadTriangleMesh    (xml);
       else if (xml->name == "QuadMesh"        ) node = state.sceneMap[id] = loadQuadMesh        (xml);
@@ -1471,6 +1526,7 @@ namespace embree
         node = state.sceneMap[id] = loadCurves(xml,type);
       }
       else if (xml->name == "PerspectiveCamera") node = state.sceneMap[id] = loadPerspectiveCamera(xml);
+      else if (xml->name == "AnimatedPerspectiveCamera") node = state.sceneMap[id] = loadAnimatedPerspectiveCamera(xml);
       else if (xml->name == "Group"           ) node = state.sceneMap[id] = loadGroupNode       (xml);
       else if (xml->name == "Transform"       ) node = state.sceneMap[id] = loadTransformNode   (xml);
       else if (xml->name == "MultiTransform"  ) node = state.sceneMap[id] = loadMultiTransformNode(xml);
