@@ -29,9 +29,7 @@ namespace embree
     ~TutorialScene();
     
     void add (Ref<SceneGraph::GroupNode> node);
-    unsigned addGeometry(Ref<SceneGraph::Node> node);
     unsigned materialID(Ref<SceneGraph::MaterialNode> material);
-    unsigned geometryID(Ref<SceneGraph::Node> geometry);
     void print_camera_names ();
     Ref<SceneGraph::PerspectiveCameraNode> getDefaultCamera();
     Ref<SceneGraph::PerspectiveCameraNode> getCamera(const std::string& name);
@@ -40,6 +38,6 @@ namespace embree
     std::vector<Ref<SceneGraph::PerspectiveCameraNode>> cameras;  //!< list of all cameras
     std::vector<Ref<SceneGraph::MaterialNode>> materials; //!< list of materials
     std::vector<Ref<SceneGraph::Node> > geometries;   //!< list of geometries
-    std::vector<Ref<SceneGraph::Light>> lights;       //!< list of lights
+    std::vector<Ref<SceneGraph::LightNode>> lights;       //!< list of lights
   };
 }
