@@ -1,7 +1,8 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
 
 namespace embree
 {
@@ -19,7 +20,7 @@ namespace embree
     void postParseCommandLine() override
     {
       /* load default scene if none specified */
-      if (scene->size() == 0 && sceneFilename.size() == 0) {
+      if (scene_empty_post_parse()) {
         FileName file = FileName::executableFolder() + FileName("models/furBall_A.ecs");
         parseCommandLine(new ParseStream(new LineCommentFilter(file, "#")), file.path());
       }
@@ -30,5 +31,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv, "hair_geometry");
+  }
   return embree::Tutorial().main(argc,argv);
 }
