@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
 
 namespace embree
 {
@@ -18,5 +19,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv, "intersection_filter");
+  }
   return embree::Tutorial().main(argc,argv);
 }
