@@ -1,7 +1,8 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
 
 namespace embree
 {
@@ -19,5 +20,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv, "user_geometry");
+  }
   return embree::Tutorial().main(argc,argv);
 }
