@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
 
 namespace embree
 {
@@ -24,7 +25,7 @@ namespace embree
     void postParseCommandLine() override
     {
       /* load default scene if none specified */
-      if (scene->size() == 0 && sceneFilename.size() == 0) {
+      if (scene_empty_post_parse()) {
         FileName file = FileName::executableFolder() + FileName("models/cornell_box.ecs");
         parseCommandLine(new ParseStream(new LineCommentFilter(file, "#")), file.path());
       }
@@ -33,5 +34,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
-  return embree::Tutorial().main(argc,argv);
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv);
+  }
+  return embree::Tutorial().main(argc, argv);
 }
